import { VError } from "verror";
/**
 * A subclass of Error that is thrown by faast.js APIs and cloud function
 * invocations.
 * @remarks
 * `FaastError` is a subclass of
 * {@link https://github.com/joyent/node-verror | VError}, which provides an API
 * for nested error handling. The main API is the same as the standard Error
 * class, namely the {@link FaastError.message}, {@link FaastError.name}, and
 * {@link FaastError.stack} properties. In addition, the
 * {@link FaastError.fullStack} property provides a more detailed stack trace.
 *
 * `FaastError` can be returned in two situations. In the first situation,
 * faast.js itself is reporting an error when it encounters a situation it
 * cannot handle. In the second situation, the user's cloud function is throwing
 * an error (or returning a rejected `Promise`). In the second situation, the
 * original error class is not re-created on the client side because faast.js
 * cannot reliably instantiate foreign subclasses of `Error`. Instead, all of
 * the properties of the `Error` on the remote side are serialized and sent to
 * the local side. These properties are available on the {@link FaastError.info}
 * object. In addition, {@link FaastError.functionName} is the name of the
 * function that was invoked, and {@link FaastError.args} are the arguments used
 * to invoke the function that caused the error.
 *
 * If available, {@link FaastError.logUrl} will be a URL pointing to the logs
 * for the specific invocation that caused the error. The `logUrl` will be
 * appended to the end of {@link FaastError.message} and
 * {@link FaastError.fullStack} as well. `logUrl` be surrounded by whilespace on
 * both sides to ease parsing as a URL by IDEs.
 * @public
 */
export declare class FaastError extends VError {
    /** Always `"FaastError"` */
    name: string;
    /**
     * The log URL if this `FaastError` was caused by a cloud function
     * invocation exception. The url will have a space at the beginning and end.
     */
    logUrl?: string;
    /**
     * The error message. If nested errors occurred, this message summarizes all
     * nested errors separated with colons (:).
     */
    message: string;
    /**
     * Error code, if any.
     */
    code?: string;
    /**
     * The stack trace for the error without include traces for underlying
     * errors. A more complete stack trace is available under
     * {@link FaastError.fullStack}.
     */
    stack: string | undefined;
    /**
     * The full stack trace including stack traces for underlying causes, if
     * any.
     */
    readonly fullStack: string;
    /**
     * Get additional metadata about this error. For errors returned from a
     * remote cloud function call, the returned object will contain all of the
     * properties from the original error on the remote side.
     */
    readonly info: {
        [key: string]: any;
    };
    /**
     * The underlying cause of the error, if any. This cause is also reflected
     * in {@link FaastError.message} and {@link FaastError.fullStack}.
     */
    cause(): Error | undefined;
    /**
     * For errors returned from a cloud function call, this is the function name
     * that was invoked that caused the error.
     */
    functionName?: string;
    /**
     * For errors returned from a cloud function call, these are the arguments
     * that caused the failed invocation.
     */
    args?: any[];
}
export declare function synthesizeFaastError(errObj: any, logUrl?: string, functionName?: string, args?: any[]): FaastError;
export declare function assertNever(x: never): never;
